{-------------------------------------------------------------------------}
{                    SC.INC     Screen Counter                            }
{-------------------------------------------------------------------------}
{                Copyright 1985                                           }
{                               by Lynn A. Canning                        }
{                                  9107 Grandview Dr.                     }
{                                  Overland Park, Ks 66212                }
{-------------------------------------------------------------------------}
const
  Menu = 'X = Exit   Line     Column     HTrip    VTrip    F2 = Trip Reset  F1 = Help ';
type
  char_cell   = record
                  code : char;
                  attr : byte;
                  end;

  screen_type = array[1..25] of array[1..80] of char_cell;

var
  ch                : char;
  i,j,l,m,n,o,b,f,t : byte;
  cx,cy             : byte;
  htrip,vtrip       : byte;
  saveattr          : byte;
  cursor_color      : byte;
  screensc          : screen_type;
  real_screen       : ^screen_type;
  previous          : boolean;
  counter_color     : byte;
  menu_color        : byte;

procedure update_screen(y,lines : byte);

  begin
  if video_mode <> 7 then
    repeat until (port[$3da] and 8) = 8;      {is raster in vertical retrace}
  if video_mode <> 7 then
    port[$3d8] := 1;                          {disable video signal}
  move(screensc[y],real_screen^[y],lines * 160); {write to screen memory}
  if video_mode <> 7 then
    port[$3d8] := 9;                          {enable video signal}
  end;

procedure read_screen(y,lines : byte);

  begin
  if video_mode <> 7 then
    repeat until (port[$3da] and 8) = 8;
  if video_mode <> 7 then
    port[$3d8] := 1;
  move(real_screen^[y],screensc[y],lines * 160); {read screen memory}
  if video_mode <> 7 then
    port[$3d8] := 9;
  end;

procedure move_cursor;
  begin
    if (i = 0) or (i = 25) then begin
      i := l;
      j := m;
      htrip := n;
      vtrip := o;
      write(Chr(7));  {no screen wrap allowed, ring bell}
      end
    else begin
      screensc[l,m].attr := saveattr;
      saveattr := screensc[i,j].attr;
      screensc[i,j].attr := cursor_color;
      update_screen(i,1);
      update_screen(l,1);
    end
end;

procedure update_menu;     {update the menu counters}
begin
  textbackground(Brxx);
  gotoxy(17,25);
  write(i:2);
  gotoxy(28,25);
  write(j:2);
  gotoxy(38,25);
  if htrip = 0 then
    htrip := j;
  write(htrip:2);
  gotoxy(47,25);
  if vtrip = 0 then
    vtrip := i;
  write(vtrip:2);
  end;

procedure CheckInput;
begin

  l := i;
  m := j;
  n := htrip;
  o := vtrip;

  Read(Kbd,Ch);

  if Ch = chr(27)  then begin                 {check for cursor keys -(ESC x)}
    previous := True;
    Read(Kbd,Ch);
  end
  else previous := False;

  if previous and (Ch = '<') then begin      {check for F2 key, if found }
    htrip := 1;                               {reset htrip counter}
    vtrip := 1;
    update_menu;
    end;

  if previous and (Ch = ';') then begin      {check for F1 key, if found }
    Open(1);
    ClrScr;
    Writexy('  Screen Counter                          Written by Lynn A. Canning',t,1,2,0);
    Writexy('  Version 4.10                                       9107 Grandview',t,1,3,0);
    Writexy('                                                     Overland Park, Ks 66212',t,1,4,0);
    Writexy('         WELCOME TO THE SCREEN COUNTER HELP MENU',t,1,5,0);
    Writexy('                  CURSOR MOVEMENT KEYS',t,1,7,0);
    Writexy('  Arrow Keys - move one position in specified direction',t,1,8,0);
    Writexy('  Home Key   - move to beginning of current line',t,1,9,0);
    Writexy('  End Key    - move to end of current line',t,1,10,0);
    Writexy('  Cntl End   - move to beginning of last line',t,1,11,0);
    Writexy('  Cntl Home  - move to beginning of first line',t,1,12,0);
    Writexy('  Screen Wrap is allowed except at columns 1 and 80',t,1,13,0);
    Writexy('                  USE OF TRIP COUNTERS',t,1,15,0);
    Writexy('  Move cursor to the first position to be counted, reset Trip Counters,',t,1,16,0);
    Writexy('  move cursor to the last position to be counted.',t,1,17,0);
    Writexy('  F2 - resets Trip Counters',t,1,18,0);
    Writexy('  X - exits program Screen Counter',t,1,19,0);
    Writexy('  Note that the Column and Line Counters are the X & Y screen coordinates.',t,1,20,0);
    Writexy('               Press Any Key To Continue',t,1,22,0);
    repeat until keypressed;
    Remove(1);
    Textcolor(counter_color);
    end;

  if previous  and (Ch = 'M') then begin      {check for right arrow (ESC M)}
    if j = 80 then begin
      j := 1;
      htrip := 1;
      vtrip := vtrip+1;
      i := i+1;
      end
    else begin
      j := j+1;
      htrip := htrip+1;
    end;
  move_cursor;
  update_menu;
  end;

  if previous and (Ch = 'K') then begin        {check for left arrow (ESC K)}
    if j = 1 then begin
      j := 80;
      htrip := j;
      vtrip := vtrip-1;
      i := i-1;
      end
    else begin
      j := j-1;
      htrip := htrip-1;
    end;
  move_cursor;
  update_menu;
  end;

  if previous and (Ch = 'P') then begin        {check for down arrow (ESC P)}
    if i = 24 then begin
      i := 1;
      vtrip := i;
      end
    else begin
      i := i+1;
      vtrip := vtrip+1;
      end;
  move_cursor;
  update_menu;
  end;

  if previous and (Ch = 'H') then begin        {check for up arrow (ESC H)}
    if i = 1 then begin
      i := 24;
      vtrip := i;
      end
    else begin
      i := i-1;
      vtrip := vtrip-1;
      end;
  move_cursor;
  update_menu;
  end;

  if previous and (Ch = 'G') then begin        {check for home key (ESC G)}
    j := 1;
    htrip := 1;
  move_cursor;
  update_menu;
  end;

  if previous and (Ch = 'O') then begin        {check for end key (ESC O)}
    j := 80;
    htrip := 80;
  move_cursor;
  update_menu;
  end;

  if previous and (Ch = 'u') then begin        {check for cntl end key (ESC u)}
    i := 24;
    j := 1;
    htrip := 1;
    vtrip := i;
  move_cursor;
  update_menu;
  end;

  if previous and (Ch = 'w') then begin       {check for cntl home key (ESC w)}
    i := 1;
    j := 1;
    htrip := 1;
    vtrip := 1;
  move_cursor;
  update_menu;
  end;

end;

{-------------------------------------------------------------------------}
{                               Main Program                              }
{-------------------------------------------------------------------------}
begin
  Initialize;
  video_mode := Mem[0000:$0449];
  if video_mode = 7 then           {7 is monochrome 3 is color graphics 80X25}
    real_screen := ptr($b000,0)    {set page offset address for monochrome}
  else
    real_screen := ptr($b800,0);   {set page offset address for color graphics}
  SetVideo;    {reset video_mode to allow color changes on composite monitors}
  if video_mode = 7 then begin
    cursor_color := 240;           {black on white}
    counter_color := 0;            {black}
    menu_color := 15;              {white}
    b := 112;
    f := 112;
    t := 112;
    end
  else begin;
    cursor_color := BlBrxx;            {blue on brown}
    counter_color := Blxx;            {blue}
    menu_color := Rxx;               {red}
    b := BkGxx;
    f := BkBrxx;
    t := BkBrxx;
    end;
  SetWindow(1,0,0,79,24,b,f,2,0,'',0,0,2);
  fillchar(screensc,4000,0);
  cy := wherey;
  cx := wherex;
  I := cy;
  J := cx;
  htrip := J;
  if I = 25 then
    I := 24;
  vtrip := I;
  read_screen(1,25);
  saveattr := screensc[i,j].attr;
  screensc[i,j].attr := cursor_color;
  update_screen(I,1);
  gotoxy(1,25);
  textcolor(menu_color);
  write(menu);
  textcolor(counter_color);
  update_menu;

  repeat
    CheckInput;
  until UpCase(Ch) = 'X';
  screensc[i,j].attr := saveattr;
  textbackground(black);
  update_screen(1,25);
  GOTOXY(cx,cy);
end;